package org.objectweb.jinha.actions;

import java.util.List;

import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;
import org.objectweb.jinha.editor.Editor;
import org.objectweb.jinha.editparts.AttributeListEditPart;
import org.objectweb.jinha.editparts.DiagramElementEditPart;
import org.objectweb.jinha.editparts.EntityEditPart;
import org.objectweb.jinha.editparts.MessageDrivenEditPart;
import org.objectweb.jinha.editparts.SessionEditPart;
import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.ModelCreationFactory;

 
/**
 * this class is used for creating attributes in a contextual ejb menu
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateAttributeAction extends SelectionAction {

    /**
     *  <code>ID</code> the id of action
     */
    public static final String ID = "create attribute action";


    /**
     * @param part
     */
    public CreateAttributeAction(IWorkbenchPart part) {
        super(part);
    }

    

    /**
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     * initialize the action
     */
    public void init() {
        setId(ID);
        setText("create attribute");
    }

 


    /** 
     *set the state enabled of the action to true or false
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     * @return the state of the action
     */
    protected boolean calculateEnabled() {
        List list = getSelectedObjects();
        if (list.size() == 1) {
            Object obj = list.get(0);
            if (obj instanceof DiagramElementEditPart) {
                return true;
            }
        }
        return false;
    }

   

    
    /** 
     * launch the action of creating an attribute in a ejb
     * @see org.eclipse.jface.action.IAction#run()
     * 
     */
    public void run() {
        System.out.println("action CreateAttributeAction performed");
        List list = getSelectedObjects();
        if (list.size() == 1) {
            Object obj = list.get(0);
            if (obj instanceof SessionEditPart) {
                SessionEditPart part = (SessionEditPart) obj;
                AttributeListEditPart attribpart = part
                        .getAttributeListEditPart();
                CreateRequest req = new CreateRequest();
                req.setType(RequestConstants.REQ_CREATE);
                req.setFactory(new ModelCreationFactory(AttributeModel.class,Editor.getModelManager().getResource()));
                Command command = attribpart.getCommand(req);
                if (command != null && getCommandStack() != null)
                    getCommandStack().execute(command);
            }
            else if (obj instanceof EntityEditPart) {
                EntityEditPart part = (EntityEditPart) obj;
                AttributeListEditPart attribpart = part
                        .getAttributeListEditPart();
                CreateRequest req = new CreateRequest();
                req.setType(RequestConstants.REQ_CREATE);
                req.setFactory(new ModelCreationFactory(AttributeModel.class,Editor.getModelManager().getResource()));
                Command command = attribpart.getCommand(req);
                if (command != null && getCommandStack() != null)
                    getCommandStack().execute(command);
            }
            else if (obj instanceof MessageDrivenEditPart) {
                MessageDrivenEditPart part = (MessageDrivenEditPart) obj;
                AttributeListEditPart attribpart = part
                        .getAttributeListEditPart();
                CreateRequest req = new CreateRequest();
                req.setType(RequestConstants.REQ_CREATE);
                req.setFactory(new ModelCreationFactory(AttributeModel.class,Editor.getModelManager().getResource()));
                Command command = attribpart.getCommand(req);
                if (command != null && getCommandStack() != null)
                    getCommandStack().execute(command);
            }
        }
    }

}