 package org.objectweb.jinha.actions;

import java.util.List;

import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;
import org.objectweb.jinha.editor.Editor;
import org.objectweb.jinha.editparts.CMPListEditPart;
import org.objectweb.jinha.editparts.EntityEditPart;
import org.objectweb.jinha.model.BusinessMethodModel;
import org.objectweb.jinha.model.CMPModel;
import org.objectweb.jinha.model.ModelCreationFactory;



/**
 * this class is used for creating a CMP field in a contextual entity bean menu
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateCMPAction extends SelectionAction {

    /**
     * Comment for <code>ID</code>
     */
    public static final String ID = "create cmp action";

    /**
     * @param part
     */
    public CreateCMPAction(IWorkbenchPart part) {
        super(part);
    }

    /** 
     * 
     * initialize the action
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     * 
     */
    public void init() {
        setId(ID);
        setText("create CMP field");
    }

    /**
     * set the enabled state of action to true or false
     *
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     * @return the state of the action
     */
    protected boolean calculateEnabled() {
        List list = getSelectedObjects();
        if (list.size() == 1) {
            Object obj = list.get(0);
            if (obj instanceof EntityEditPart) {
                return true;
            }
        }
        return false;
    }

    /** 
     * launch the action of creating a CMP field in a entity bean
     * @see org.eclipse.jface.action.IAction#run()
     * 
     */
    public void run() {
        System.out.println("action CreateCMPAction performed");
        List list = getSelectedObjects();
        if (list.size() == 1) {
            Object obj = list.get(0);
            EntityEditPart part = (EntityEditPart) obj;
            CMPListEditPart CMPpart = part.getCMPListEditPart();
            CreateRequest req = new CreateRequest();
            req.setType(RequestConstants.REQ_CREATE);
            req.setFactory(new ModelCreationFactory(CMPModel.class,Editor.getModelManager().getResource()));
            Command command = CMPpart.getCommand(req);
            if (command != null && getCommandStack() != null)
                getCommandStack().execute(command);
        }
    }

}