 
package org.objectweb.jinha.actions;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.objectweb.jinha.editor.Editor;
import org.objectweb.jinha.generate.Generate;
import org.objectweb.jinha.model.DiagramModel;
import org.objectweb.jinha.model.emf.jinha.ModuleEjbDefinition;

import com.objectlearn.jdt.j2ee.core.Project;


/**
 * this class is used for generating Xdoclet files with an EMF model generated with JINHa 
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class GenerateXdocletFilesAction implements IActionDelegate,
        IObjectActionDelegate {

    /**
     * Comment for <code>page</code>
     */
    private IWorkbenchPage page;

    /**
     *  <code>selectedFolder</code> is the folder where will be stored xdoclet files
     */
    private IFolder        selectedFolder;
    private IPackageFragment packageFragment;

    /**
     *  <code>editor</code> represents the Jinha editor
     */
    private Editor         editor = null;

    /**
     * activate JINHa editor
     * @param action  
     * @param part the part to activate
     */
    public void setActivePart(IAction action, IWorkbenchPart part) {
        page = part.getSite().getPage();
    }


    /**
     * called when the selection in workspace has changed 
     * @param action
     * @param selection the selected items in workspace
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        selectedFolder = null;
        packageFragment=null;
        if (selection instanceof IStructuredSelection) {
            IStructuredSelection ssel = (IStructuredSelection) selection;
            Object o = ssel.getFirstElement();
            if (isEjbModule(o)) {
                if(o instanceof IFolder) {
                    selectedFolder = (IFolder) ssel.getFirstElement();
                action.setEnabled(selectedFolder != null && getEditor()!=null);
                }
                else {
                    packageFragment=(IPackageFragment)ssel.getFirstElement();
                    action.setEnabled(packageFragment != null && getEditor()!=null);
                }
            }
            else action.setEnabled(false);
        }
        

    }

    
    /**
     * @return true if Jinha is already opened in the Workbench, else return false
     */
    public boolean isJINHaOpened() {
        if (getEditor() != null)
            return true;
        else
            return false;
    }

    /**
     * @return the JINHa instance editor
     */
    public Editor getEditor() {
        IEditorReference[] editors = page.getEditorReferences();

        int i = 0;
        boolean finded=false;
        while (i < editors.length && finded==false) {
            if(editors[i].getEditor(false) instanceof Editor) 
                    if(selectedFolder!=null 
                        && ((Editor)editors[i].getEditor(false)).getContents().getModuleName().equals(selectedFolder.getName()))
                                finded=true;
                            else if(packageFragment!=null
                                    && ((Editor)editors[i].getEditor(false)).getContents().getModuleName().equals(packageFragment/*.getJavaProject()*/.getElementName()))
                                finded=true;
 
                                
        i++;
    }
        if (finded==true)
            return (Editor) editors[i-1].getEditor(false);
        return null;
    }

    /**
     * @param o the object to be tested
     * @return true if o is an ejbModule, eslse return false
     */
    public boolean isEjbModule(Object o) {
       
        IProject p=null;
        String nameModule=null;
        if (o instanceof IFolder) {
           p= ((IFolder)o).getProject();
           nameModule=((IFolder)o).getName();
        }
        else if (o instanceof IPackageFragment) {
            nameModule=((IPackageFragment)o).getElementName().substring(((IPackageFragment)o).getElementName().lastIndexOf(".")+1);
            p= ((IPackageFragment)o).getJavaProject().getProject();
        }
        Project lombozProject = null;
        lombozProject = lombozProject.findProject(p);
        if(lombozProject.findModule(nameModule)!=null)
            return true;
        else return false;
           
    
    
    }

    /** 
     * launch the action of generating Xdoclet files in a ejbModule
     * @see org.eclipse.jface.action.IAction#run()
     * 
     */
    public void run(IAction action) {
        if (getEditor() != null) {
            ModuleEjbDefinition module = getEditor().getContents()
                    .getModuleEMF();

            if (module != null) {
                Generate generate = new Generate();
                generate.generateLombozFile(module,((DiagramModel)getEditor().getContents()).getModuleName(),((DiagramModel)getEditor().getContents()).getProjectName(),getEditor());
            }
        }
    }
}

