package org.objectweb.jinha.commands;

import java.awt.Point;
import org.eclipse.draw2d.AbsoluteBendpoint;
import org.eclipse.gef.commands.Command;
import org.eclipse.draw2d.geometry.*;
import org.objectweb.jinha.model.RelationModel;

/**
 * create and add a bendpoint to the RelationModel
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateBendpointCommand extends MainCommand {

    private org.eclipse.draw2d.geometry.Point location;

    private int                               index;

    private RelationModel                     model;

    private AbsoluteBendpoint                 bendpoint;

    /**
     * @param model the relation which will have a bendpoint
     * @param index the index where will be stored the bendpoint in the list of bendpoints of the relation
     * @param location the location where will be displayed the bendpoint
     */
    public CreateBendpointCommand(RelationModel model, int index,
            org.eclipse.draw2d.geometry.Point location) {
        this.model = model;
        this.index = index;
        this.location = location;
    }

    /**
     * performs the operation of creating and adding the bendpoint to the relation
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        bendpoint = new AbsoluteBendpoint(location);
        bendpoint.setLocation(location);
        model.addBendpoint(bendpoint, index);
    }

    /**
     * add the bendpoint to the relation
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        model.addBendpoint(bendpoint, index);
    }

    /**
     * remove the bendpoint to the relation
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        model.removeBendpoint(bendpoint);
    }
}