package org.objectweb.jinha.commands;

import java.util.Iterator;
import java.util.List;

import org.objectweb.jinha.model.BusinessMethodModel;
import org.objectweb.jinha.model.CMPModel;
import org.objectweb.jinha.model.ListModel;
import org.objectweb.jinha.model.emf.jinha.GraphicalEntity;
import org.objectweb.jinha.model.emf.jinha.GraphicalSession;

import com.sun.java.xml.ns.j2ee.MethodType;

/**
 * add a business method to a ejb and the associated emf model
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateBusinessMethodCommand extends MainCommand {

    private ListModel           list;

    private BusinessMethodModel newMethod;

    /**
     * @param list the list of business methods where will be added the newMethod
     * @param newMethod the business method to be added
     */
    public CreateBusinessMethodCommand(ListModel list,
            BusinessMethodModel newMethod) {
        this.newMethod = newMethod;
        this.list = list;
    }

    /**
     * performs the operation of adding the business method to the list of business methods of a ejb
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
    	String ident=getIdent();
        this.newMethod.setIdentifier(ident);
        this.newMethod.setReturnType("void");
        this.newMethod.setVisibility("public");
        this.newMethod.getMethod().getMethodIntf().setValue("Remote");
        this.newMethod.getMethod().getMethodName().setValue(ident);
        list.addElement(newMethod);
        if (list.getEjb() instanceof GraphicalSession)
            ((GraphicalSession) list.getEjb()).getSessionBean().getMethods()
                    .add(newMethod.getMethod());
         else if(list.getEjb() instanceof GraphicalEntity)
           ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().add(newMethod.getMethod());
        // else if(list.getEjb() instanceof GraphicalMessageDriven)
        //     ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().add(newMethod.getMethod());

    }
    /**
	 * @return
	 */
	private String getIdent() {
		String id="BusinessMethod"+getCpt();
		Iterator it=list.getElementList().iterator();
		while(it.hasNext()){
			if(id.equals(((BusinessMethodModel)it.next()).getIdentifier())){
				id+="_"+getCpt();
				it=list.getElementList().iterator();
			}
		}
		return id;
	}
    /**
     * performs execute() method
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
 //       res.getContents().add(newMethod.getMethod());
    }

    
    /**
     * remove the business method of the list
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        list.removeElement(newMethod.getMethod());
        if (list.getEjb() instanceof GraphicalSession)
            ((GraphicalSession) list.getEjb()).getSessionBean().getMethods()
                    .remove(newMethod.getMethod());
         else if(list.getEjb() instanceof GraphicalEntity)
           ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().remove(newMethod.getMethod());
        // else if(list.getEjb() instanceof GraphicalMessageDriven)
        //     ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().remove(newMethod.getMethod());

    }
    
    
    public  int getCpt() {
        int cpt=0;
        List l=null;
        if(list.getEjb() instanceof GraphicalSession) {
            l= ((GraphicalSession)list.getEjb()).getSessionBean().getMethods();
        }
        else if(list.getEjb() instanceof GraphicalEntity) {
            l= ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods();
            
        }
        Iterator i=l.iterator();
        while(i.hasNext()) {
            MethodType meth=(MethodType)i.next();
            if(meth.getMethodIntf().getValue().equals("Remote") || meth.getMethodIntf().getValue().equals("Local"))
                cpt++;
        }
        return cpt+1;
    }
}

