package org.objectweb.jinha.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.commands.Command;
import org.objectweb.jinha.model.BusinessMethodModel;
import org.objectweb.jinha.model.CMPModel;
import org.objectweb.jinha.model.HomeMethodModel;
import org.objectweb.jinha.model.ListModel;
import org.objectweb.jinha.model.emf.jinha.GraphicalEntity;
import org.objectweb.jinha.model.emf.jinha.GraphicalSession;

import com.sun.java.xml.ns.j2ee.MethodType;

/**
 * add a Home method to a ejb's list of Home methods
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateHomeMethodCommand extends MainCommand {

    private ListModel       list;

    private HomeMethodModel newMethod;

    /**
     * @param list the list where will be stored the Home method
     * @param newMethod the Home method to be added
     */
    public CreateHomeMethodCommand(ListModel list, HomeMethodModel newMethod) {
        this.newMethod = newMethod;
        this.list = list;
    }

    /**
     * perform the operation of adding the Home method to the entity's list of home methods
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
    	String ident=getIdent();
        this.newMethod.setIdentifier(ident);
        this.newMethod.setReturnType("void");
        this.newMethod.setVisibility("public");
        this.newMethod.getMethod().getMethodIntf().setValue("Home");
        this.newMethod.getMethod().getMethodName().setValue(ident);

        list.addElement(newMethod);
        if (list.getEjb() instanceof GraphicalSession)
            ((GraphicalSession) list.getEjb()).getSessionBean().getMethods()
                    .add(newMethod.getMethod());
         else if(list.getEjb() instanceof GraphicalEntity)
            ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().add(newMethod.getMethod());
        // else if(list.getEjb() instanceof GraphicalMessageDriven)
        //     ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().add(newMethod.getMethod());

    }
    /**
	 * @return
	 */
	private String getIdent() {
		String id="HomeMethod"+getCpt();
		Iterator it=list.getElementList().iterator();
		while(it.hasNext()){
			if(id.equals(((HomeMethodModel)it.next()).getIdentifier())){
				id+="_"+getCpt();
				it=list.getElementList().iterator();
			}
		}
		return id;
	}
    /**
     * same as execute()
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
  //      res.getContents().add(newMethod.getMethod());
    }

    /**
     * remove the Home method to the entity's list of home methods
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        list.removeElement(newMethod);
//        res.getContents().remove(newMethod.getMethod());
        if (list.getEjb() instanceof GraphicalSession)
            ((GraphicalSession) list.getEjb()).getSessionBean().getMethods()
                    .remove(newMethod.getMethod());
         else if(list.getEjb() instanceof GraphicalEntity)
            ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().remove(newMethod.getMethod());
        // else if(list.getEjb() instanceof GraphicalMessageDriven)
        //     ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().remove(newMethod.getMethod());

    }
    
    public  int getCpt() {
        int cpt=0;
        List l=null;
        if(list.getEjb() instanceof GraphicalSession) {
            l= ((GraphicalSession)list.getEjb()).getSessionBean().getMethods();
        }
        else if(list.getEjb() instanceof GraphicalEntity) {
            l= ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods();
            
        }
        Iterator i=l.iterator();
        while(i.hasNext()) {
            MethodType meth=(MethodType)i.next();
            if(meth.getMethodIntf().getValue().equals("Home") || meth.getMethodIntf().getValue().equals("LocalHome"))
                cpt++;
        }
        return cpt+1;
    }
}

