package org.objectweb.jinha.commands;

import org.objectweb.jinha.model.CMRModel;
import org.objectweb.jinha.model.EntityModel;
import org.objectweb.jinha.model.RelationModel;

/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class DeleteRelationCommand extends MainCommand {

    private RelationModel relation;

    private EntityModel   source;

    private EntityModel   target;

    private CMRModel      CMRsrc;

    private CMRModel      CMRtarget;

    /**
     * @param relation the relation to be deleted
     */
    public DeleteRelationCommand(RelationModel relation) {
        this.relation = relation;
    }

    /**
     * performs the deletion of the relation
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        source = relation.getSource();
        target = relation.getTarget();
        CMRsrc = relation.getCMRsrc();
        CMRtarget = relation.getCMRtarget();
        if (CMRsrc != null) {
            // CMRsrc.setRelation(null);
            source.getCMRs().removeElement(CMRsrc);
        }
        if (CMRtarget != null) {
            // CMRtarget.setRelation(null);
            target.getCMRs().removeElement(CMRtarget);
        }

        relation.setSource(null);
        relation.setTarget(null);
        relation.setCMRsrc(null);
        relation.setCMRtarget(null);

    }

    /**
     * same as execute()
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
    }

    /**
     * restore the relation
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        if (CMRsrc != null) {
            CMRsrc.setRelation(relation);
            source.getCMRs().addElement(CMRsrc);
            relation.setCMRsrc(CMRsrc);
        }

        relation.setSource(source);
        relation.setTarget(target);
        if (CMRtarget != null) {
            target.getCMRs().addElement(CMRtarget);
            CMRtarget.setRelation(relation);
            relation.setCMRtarget(CMRtarget);
        }
    }
}