package org.objectweb.jinha.editparts;
 
import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.draw2d.AutomaticRouter;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.FanRouter;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.tools.MarqueeDragTracker;
import org.objectweb.jinha.editpolicies.XYLayoutEditPolicyImpl;
import org.objectweb.jinha.model.DiagramModel;


/**
 *  the editpart for a diagram 
 * catch event from the model and eventually represent it to the figure
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class DiagramEditPart extends ObjectEditPart {

    // prevent from graphical collisions between object
    private AutomaticRouter router;

   
    /**
     * @return the figure associated to a diagram
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    public IFigure createFigure() {
        Figure f = new FreeformLayer();
        f.setLayoutManager(new FreeformLayout());
        return f;
    }

    
    /**
     * @return the lsit of childre for this part
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     */
    public List getModelChildren() {
        DiagramModel model = (DiagramModel) getModel();
        return model.getElements();
    }

    /**
     *  defines allowed policies for this part
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    public void createEditPolicies() {
        super.createEditPolicies();
        this.installEditPolicy(EditPolicy.LAYOUT_ROLE,
                new XYLayoutEditPolicyImpl());
    }

    /**
     * @param request 
     * @return
     * @see org.eclipse.gef.EditPart#getDragTracker(org.eclipse.gef.Request)
     */
    public DragTracker getDragTracker(Request request) {
        return new MarqueeDragTracker();
    }

    
    /**
     * @param ev event caught and treated by this part 
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent ev) {
        super.propertyChange(ev);
        if (ev.getPropertyName().equals(DiagramModel.PROPERTY_ELEMENT_ADDED))
            refreshChildren();
        else if (ev.getPropertyName().equals(
                DiagramModel.PROPERTY_ELEMENT_REMOVED))
            refreshChildren();
    }

    
    /**
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    public void refreshVisuals() {
        ConnectionLayer cLayer = (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
        cLayer.setConnectionRouter(getConnectionRouter());
    }

    
    /**
     * @return
     */
    public AutomaticRouter getConnectionRouter() {
        if (router == null) {
            router = new FanRouter();
            router.setNextRouter(new BendpointConnectionRouter());
        }
        return router;
    }
}