package org.objectweb.jinha.editparts;
 
import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.tools.DragEditPartsTracker;
import org.objectweb.jinha.editpolicies.ComponentEditPolicyForwardDelete;
import org.objectweb.jinha.model.DiagramElementModel;


/**
 *  the editpart for a diagram element
 * catch event from the model and eventually represent it to the figure
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public abstract class DiagramElementEditPart extends ObjectEditPart {

    
    /**
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    public void refreshVisuals() {
        super.refreshVisuals();
        refreshParentLayout();
    }

    
    /**
     * @param request request to treat with an edit part in param
     * @return the dragTracker associated
     * @see org.eclipse.gef.EditPart#getDragTracker(org.eclipse.gef.Request)
     */
    public DragTracker getDragTracker(Request request) {
        return new DragEditPartsTracker(this);
    }

    
    /**
     * @param ev event caught and treated by this part
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent ev) {
        super.propertyChange(ev);
        if (ev.getPropertyName().equals(
                DiagramElementModel.PROPERTY_CONSTRAINT_CHANGED))
            refreshParentLayout();
    }

    
    /**
     * 
     */
    private void refreshParentLayout() {
        DiagramElementModel model = (DiagramElementModel) getModel();
        Rectangle constraint = model.getConstraint().getCopy();
        DiagramEditPart parent = (DiagramEditPart) getParent();
        IFigure parentContentPane = parent.getContentPane();
        LayoutManager parentLayoutManager = parentContentPane
                .getLayoutManager();
        parentLayoutManager.setConstraint(getFigure(), constraint);
        parentContentPane.revalidate();
    }

    /**
     * 
     */
    public void createEditPolicies() {
        super.createEditPolicies();
        installEditPolicy(EditPolicy.COMPONENT_ROLE,
                new ComponentEditPolicyForwardDelete() {
                });
    }
}