package org.objectweb.jinha.editpolicies;

import java.util.Iterator;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.GroupRequest;
import org.objectweb.jinha.commands.CreateAttributeCommand;
import org.objectweb.jinha.commands.CreateBusinessMethodCommand;
import org.objectweb.jinha.commands.CreateCMPCommand;
import org.objectweb.jinha.commands.CreateCMRCommand;
import org.objectweb.jinha.commands.CreateFinderCommand;
import org.objectweb.jinha.commands.CreateHomeMethodCommand;
import org.objectweb.jinha.commands.ListElementAddCommand;
import org.objectweb.jinha.commands.ListElementChangeConstraintCommand;
import org.objectweb.jinha.commands.ListElementDeleteCommand;
import org.objectweb.jinha.commands.ListElementOrphanCommand;
import org.objectweb.jinha.editparts.AttributeEditPart;
import org.objectweb.jinha.editparts.AttributeListEditPart;
import org.objectweb.jinha.editparts.BusinessMethodEditPart;
import org.objectweb.jinha.editparts.BusinessMethodListEditPart;
import org.objectweb.jinha.editparts.CMPEditPart;
import org.objectweb.jinha.editparts.CMPListEditPart;
import org.objectweb.jinha.editparts.CMRListEditPart;
import org.objectweb.jinha.editparts.FinderEditPart;
import org.objectweb.jinha.editparts.FinderListEditPart;
import org.objectweb.jinha.editparts.HomeMethodEditPart;
import org.objectweb.jinha.editparts.HomeMethodListEditPart;
import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.BusinessMethodModel;
import org.objectweb.jinha.model.CMPModel;
import org.objectweb.jinha.model.CMRModel;
import org.objectweb.jinha.model.FinderModel;
import org.objectweb.jinha.model.HomeMethodModel;
import org.objectweb.jinha.model.ListModel;

/**
 * this policy manage each list in an ejb, for moving elements (methods, CMP, CMR...)inside the list or between lists
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class ToolbarLayoutEditPolicyImpl extends ToolbarLayoutEditPolicy
{

	/**
	 * @param child
	 * @param after
	 * @return
	 * @see org.eclipse.gef.editpolicies.OrderedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, org.eclipse.gef.EditPart)
	 */
	protected Command createAddCommand(EditPart child, EditPart after)
	{
		if ((getHost() instanceof AttributeListEditPart
			&& child instanceof AttributeEditPart)
			|| (getHost() instanceof BusinessMethodListEditPart
				&& child instanceof BusinessMethodEditPart)
            || (getHost() instanceof HomeMethodListEditPart
                && child instanceof HomeMethodEditPart)
            || (getHost() instanceof CMPListEditPart
                && child instanceof CMPEditPart)
           /* || (getHost() instanceof CMRListEditPart
                && child instanceof CMREditPart)*/
            || (getHost() instanceof FinderListEditPart
                && child instanceof FinderEditPart))
            
		{
			Object childModel = child.getModel();
			Object afterModel = (after == null) ? null : after.getModel();
			ListModel list = (ListModel) getHost().getModel();
			return new ListElementAddCommand(list, childModel, afterModel);
		}
		else
			return null;
	}
	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getOrphanChildrenCommand(org.eclipse.gef.Request)
	 */
	protected Command getOrphanChildrenCommand(Request request)
	{
		CompoundCommand cc = new CompoundCommand();
		ListModel list = (ListModel) getHost().getModel();
		Iterator it = ((GroupRequest) request).getEditParts().iterator();
		while (it.hasNext())
		{
			cc.add(
				new ListElementOrphanCommand(
					list,
					((EditPart) it.next()).getModel()));
		}
		return cc;
	}
	/**
	 * @param child
	 * @param after
	 * @return
	 * @see org.eclipse.gef.editpolicies.OrderedLayoutEditPolicy#createMoveChildCommand(org.eclipse.gef.EditPart, org.eclipse.gef.EditPart)
	 */
	protected Command createMoveChildCommand(EditPart child, EditPart after)
	{
		Object childModel = child.getModel();
		Object afterModel = (after == null) ? null : after.getModel();
		ListModel list = (ListModel) getHost().getModel();
		return new ListElementChangeConstraintCommand(
			list,
			childModel,
			afterModel);
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request)
	{
		if (getHost() instanceof BusinessMethodListEditPart)
		{
			Object newObj = request.getNewObject();
			if (newObj instanceof BusinessMethodModel)
			{
				BusinessMethodModel newMethod = (BusinessMethodModel) newObj;
				ListModel list = (ListModel) getHost().getModel();
				return new CreateBusinessMethodCommand(list, newMethod);
			}
		}
		else if(getHost() instanceof AttributeListEditPart)
		{
			Object newObj = request.getNewObject();
			if (newObj instanceof AttributeModel)
			{
				AttributeModel newAttribute = (AttributeModel) newObj;
				ListModel list = (ListModel) getHost().getModel();
				return new CreateAttributeCommand(list, newAttribute);
			}
		}
        else if(getHost() instanceof CMPListEditPart)
        {
            Object newObj = request.getNewObject();
            if (newObj instanceof CMPModel)
            {
                CMPModel newCMP = (CMPModel) newObj;
                ListModel list = (ListModel) getHost().getModel();
                return new CreateCMPCommand(list, newCMP);
            }
        }
        else if(getHost() instanceof CMRListEditPart)
        {
            Object newObj = request.getNewObject();
            if (newObj instanceof CMRModel)
            {
                CMRModel newCMR = (CMRModel) newObj;
                ListModel list = (ListModel) getHost().getModel();
                return new CreateCMRCommand(list, newCMR);
            }
        }
        else if(getHost() instanceof FinderListEditPart)
        {
            Object newObj = request.getNewObject();
            if (newObj instanceof FinderModel)
            {
                FinderModel newFinder = (FinderModel) newObj;
                ListModel list = (ListModel) getHost().getModel();
                return new CreateFinderCommand(list, newFinder);
            }
        }
        else if(getHost() instanceof HomeMethodListEditPart)
        {
            Object newObj = request.getNewObject();
            if (newObj instanceof HomeMethodModel)
            {
                HomeMethodModel newMethod = (HomeMethodModel) newObj;
                ListModel list = (ListModel) getHost().getModel();
                return new CreateHomeMethodCommand(list, newMethod);
            }
        }
		return null;
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	protected Command getDeleteDependantCommand(Request request)
	{
		CompoundCommand cc = new CompoundCommand();
		ListModel list = (ListModel) getHost().getModel();
		Iterator it = ((GroupRequest) request).getEditParts().iterator();
		while (it.hasNext())
		{
			cc.add(
				new ListElementDeleteCommand(
					list,
					((EditPart) it.next()).getModel()));
		}
		return cc;
	}

}
