package org.objectweb.jinha.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import com.sun.java.xml.ns.j2ee.CmrFieldType;
import com.sun.java.xml.ns.j2ee.MethodType;

/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CMRModel extends ObjectModel
{
    private RelationModel relation;
    private String multiplicity;
    private String returnType;
    private String identifier;
    private List parametersTypes;
    private String roleName;
    private String visibility;

	private IPropertyDescriptor[] descriptors;
	private CmrFieldType cmrFieldType=null;
   

	public static final String PROPERTY_SIGNATURE_CHANGED = "signature";
    public static final String PUBLIC = "public";
    public static final String PRIVATE = "private";
    public static final String PROTECTED = "protected";

	public static final String ID_PROPERTY_IDENTIFIER = "identifier";
	public static final String ID_PROPERTY_RETURN_TYPE = "return type";
	public static final String ID_PROPERTY_VISIBILITY = "visibility";
     public static final String    ID_PROPERTY_MULTIPLICITY      = "multiplicity";

     public static final String ID_PROPERTY_ROLE_NAME = "role-name";
     public static final String    PROPERTY_MULTIPLICITY_CHANGED = "multiplicity";
     
     public static final String[] manyChoiceReturn = {"java.util.Set","java.util.Collection"};

    /**
     * 
     */
    public CMRModel(Resource res)
	{
        factory.setTargetClass(CmrFieldType.class);
        factory.setRes(res);
        cmrFieldType=(CmrFieldType)factory.getNewObject();
		setSignature(PUBLIC, "no_type", "no_identifier", new ArrayList());
		setMultiplicity("one");
		
		 setRoleName("");
    }
    
    //  Create a basic CMRModel with the name of the Ejb Target
    public CMRModel(String ejbTarget, Resource res) {
        factory.setTargetClass(MethodType.class);
        factory.setRes(res);
        ///method = (MethodType) factory.getNewObject();
        setSignature(PUBLIC, ejbTarget, "get"+ejbTarget, new ArrayList());
        setMultiplicity("one");
        
        setRoleName("");
    }
    
	/**
	 * @param multiplicity
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parametersTypes
	 */
	public CMRModel(String multiplicity,String visibility, String returnType, String identifier, List parametersTypes,Resource res)
	{
        setMultiplicity(multiplicity);
        setSignature(visibility, returnType, identifier, parametersTypes);
	}
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parametersTypes
	 */
	public void setSignature(String visibility, String returnType, String identifier, List parametersTypes)
	{
		this.visibility = visibility;
		this.returnType = returnType;
		this.identifier = identifier;
		this.parametersTypes = parametersTypes;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	
    }
	/**
	 * @return
	 */
	public String getVisibility()
	{
		return visibility;
	}
	/**
	 * @return
	 */
	public String getReturnType()
	{
		return returnType;
	}
	/**
	 * @return
	 */
	public String getIdentifier()
	{
		return identifier;
	}
	/**
	 * @return
	 */
	public List getParametersTypes()
	{
		//return parametersTypes;
        return null;
	}
	/**
	 * @return
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		String txt = returnType + " " + identifier + "(";
		Iterator it = parametersTypes.iterator();
		if (it.hasNext())
			txt += it.next().toString();
		while (it.hasNext())
		{
			txt += " ," + it.next().toString();
		}
		txt += ")";
		return txt;
	}
	/**
	 * @param returnType
	 */
	public void setReturnType(String returnType)
	{
		this.returnType = returnType;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @param identifier
	 */
	public void setIdentifier(String identifier)
	{
		this.identifier = identifier;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @param visibility
	 */
	public void setVisibility(String visibility)
	{
        if (visibility.equals("public") || visibility.equals("protected") || visibility.equals("private"))
		{
			this.visibility = visibility;
			firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
		}
	}
	/**
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null || getMultiplicity() == "one")
		{
            descriptors = (new IPropertyDescriptor[] {
				new PropertyDescriptor(ID_PROPERTY_RETURN_TYPE, "return type"),
				new PropertyDescriptor(ID_PROPERTY_IDENTIFIER, "identifier"),
				new ComboBoxPropertyDescriptor(
					ID_PROPERTY_VISIBILITY,
					"visibility",
					new String[] { "public", "protected", "private" }),
                new ComboBoxPropertyDescriptor(
                    ID_PROPERTY_MULTIPLICITY,
                    "multiplicity",
                    new String[] { "one", "many" }),
                    new TextPropertyDescriptor(ID_PROPERTY_ROLE_NAME,ID_PROPERTY_ROLE_NAME)
                    });
		}
        
        else if(getMultiplicity() == "many")
        {
            descriptors = (new IPropertyDescriptor[] {
                    new ComboBoxPropertyDescriptor(ID_PROPERTY_RETURN_TYPE, "return type",manyChoiceReturn),
                    new PropertyDescriptor(ID_PROPERTY_IDENTIFIER, "identifier"),
                    new ComboBoxPropertyDescriptor(
                        ID_PROPERTY_VISIBILITY,
                        "visibility",
                        new String[] { "public", "protected", "private" }),
                    new ComboBoxPropertyDescriptor(
                        ID_PROPERTY_MULTIPLICITY,
                        "multiplicity",
                        new String[] { "one", "many" }),
                        new TextPropertyDescriptor(ID_PROPERTY_ROLE_NAME,ID_PROPERTY_ROLE_NAME)
                        });
        }
		return descriptors;
	}
	/**
	 * @param propName
	 * @param val
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object propName, Object val)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
            {
                if(getMultiplicity() == "many")
                {
                    setReturnType(manyChoiceReturn[((Integer) val).intValue()]);
                }
                else
                {
                    String targetName = getIdentifier().substring(3,identifier.length());
                    setReturnType(targetName);
                }
                    
            }
				
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				setIdentifier(val.toString());
            else if (((String) propName).equals(ID_PROPERTY_ROLE_NAME))
                setRoleName(val.toString());
            else if (((String) propName).equals(ID_PROPERTY_MULTIPLICITY))
            {
                if (((Integer) val).intValue() == 0)
                    setMultiplicity("one");
                else if (((Integer) val).intValue() == 1)
                    setMultiplicity("many");
            }
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (((Integer) val).intValue() == 0)
					setVisibility(PUBLIC);
				else if (((Integer) val).intValue() == 2)
					setVisibility(PRIVATE);
				else if (((Integer) val).intValue() == 1)
					setVisibility(PROTECTED);
			}
		}
		else
			super.setPropertyValue(propName, val);
	}
	/**
	 * @param propName
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object propName)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
            {
                if(getMultiplicity()== "many")
                {
                    if(getReturnType() == manyChoiceReturn[0])
                        return new Integer(0);
                    else if(getReturnType() == manyChoiceReturn[1])
                        return new Integer(1);
                    
                    else 
                    {
                        setReturnType(manyChoiceReturn[0]);
                        return new Integer(0);
                    }
                        
                }
                String targetName = getIdentifier().substring(3,identifier.length());
                setReturnType(targetName);
                return getReturnType();
            }
				
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				return identifier;
            else if (((String) propName).equals(ID_PROPERTY_ROLE_NAME))
                return getRoleName();
            else if (((String) propName).equals(ID_PROPERTY_MULTIPLICITY))
            {
                if(getMultiplicity().equals("one"))
                    return new Integer(0);
                else
                    return new Integer(1);
            }
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (visibility == PUBLIC)
					return new Integer(0);
				else if (visibility == PRIVATE)
					return new Integer(2);
				else if (visibility == PROTECTED)
					return new Integer(1);
				else
					return super.getPropertyValue(propName);
			}
			else
				return super.getPropertyValue(propName);
		}
		else
			return super.getPropertyValue(propName);
	}
    /**
     * @return Returns the relation.
     */
    public RelationModel getRelation() {
        return relation;
    }
    /**
     * @param relation The relation to set.
     */
    public void setRelation(RelationModel relation) {
        this.relation = relation;
    }
    /**
     * @return Returns the multiplicity.
     */
    public String getMultiplicity() {
        return multiplicity;
    }
    /**
     * @param multiplicity The multiplicity to set.
     */
    public void setMultiplicity(String multiplicity) {
        this.multiplicity = multiplicity;
        firePropertyChange(PROPERTY_MULTIPLICITY_CHANGED, null, null);
    }
    /**
     * @return Returns the roleName.
     */
    public String getRoleName() {
        return roleName;
    }
    /**
     * @param roleName The roleName to set.
     */
    public void setRoleName(String roleName) {
        this.roleName = roleName;
    }
    /**
     * @return Returns the cmrFieldType.
     */
    public CmrFieldType getCmrFieldType() {
        return cmrFieldType;
    }
    /**
     * @param cmrFieldType The cmrFieldType to set.
     */
    public void setCmrFieldType(CmrFieldType cmrFieldType) {
        this.cmrFieldType = cmrFieldType;
    }
}