
package org.objectweb.jinha.model;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.ui.IWorkbenchPage;
import org.objectweb.jinha.editor.Editor;
import org.objectweb.jinha.model.emf.jinha.JinhaFactory;
import org.objectweb.jinha.model.emf.jinha.JinhaPackage;
import org.objectweb.jinha.model.emf.jinha.ModuleEjbDefinition;
import org.objectweb.jinha.model.emf.jinha.impl.JinhaPackageImpl;


/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class EMFModelManager {
	/**
	 * For the purpose of the simple editor, a file can only contain a network.
	 * In EMF, a resource provides the way to have access to the model content.
	 */
	private   Resource resource = null;

	/**
	 * Contains the factory associated with the model.
	 */
	private  JinhaFactory JinhaFactory = null;

	/**
	 * Gives access to the top level Jinha contained in the resource.
	 */
	private  ModuleEjbDefinition module = null;
	/**
	 * @return Returns the editor.
	 */
	public static Editor getEditor() {
		return editor;
	}
	/**
	 * @param editor The editor to set.
	 */
	public void setEditor(Editor editor) {
		this.editor = editor;
	}

	private static IWorkbenchPage page=null;
	private static Editor editor=null;
	/**
	 * Returns the resource containing the Jinha. 
	 * @param path
	 * @return
	 */
	public Resource getResource(URI path) {
		if (resource == null) {
          // JinhaResourceFactoryImpl jinhaResourceFactoryImpl = new JinhaResourceFactoryImpl();
			//resource =jinhaResourceFactoryImpl.createResource(path);
            ResourceSet resSet=getResourceSet();
            resource=resSet.createResource(path);

        }
		return resource;
	}

	/**
	 * Creates a resource to contain the module. 
	 * @param path
	 * @return
	 */
	private Resource createResource(URI path) {
		//if (resource == null) {
	        resource=null;
            ResourceSet resSet=new ResourceSetImpl();
            resource=resSet.createResource(path);           
      //  }
		return resource;
	}

	/**
	 * Returns the resource set.
	 * @param 
	 * @return
	 */
	private ResourceSet getResourceSet() {
		// Initialize the Jinha package
		JinhaPackageImpl.init();
		// Register the XMI resource factory for the .Jinha extension
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map m = reg.getExtensionToFactoryMap();
		m.put("jinha", new XMIResourceFactoryImpl());
        m.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		// Obtain a new resource set
		return new ResourceSetImpl();
	}

	/**
	 * Returns the factory associated with the model
	 * @return
	 */
	 public JinhaFactory getFactory() {
		if (JinhaFactory == null) {
			// Access the factory (needed to create instances)
			Map registry = EPackage.Registry.INSTANCE;
			String JinhaURI = JinhaPackage.eNS_URI;
			JinhaPackage JinhaPackage =
				(JinhaPackage) registry.get(JinhaURI);
			JinhaFactory = JinhaPackage.getJinhaFactory();
		}
		return JinhaFactory;
	}

	/**
	 * Creates a new ModuleEjbDefinition.
	 * @param 
	 * @return
	 */
	public ModuleEjbDefinition createModule(URI path) {
		createResource(path);
		// Create a new Jinha model
		Map registry = EPackage.Registry.INSTANCE;
		String JinhaURI = JinhaPackage.eNS_URI;
		JinhaPackage nPackage = (JinhaPackage) registry.get(JinhaURI);
		JinhaFactory nFactory = nPackage.getJinhaFactory();
		module = nFactory.createModuleEjbDefinition();
		resource.getContents().add(module);
		return module;
	}

	/**
	 * Loads the content of the model from the file.
	 * @param path
	 */
	public void load(URI path) throws IOException {
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
        options.put(org.eclipse.emf.ecore.xmi.XMLResource.OPTION_ENCODING, "UTF-8");
		resource.load(options);
	}

    /**
     * reloads the content of the model from the file.
     * @param path
     */
    public void reload(URI path) throws IOException {
        getResource(path).unload();
        load(path);
    }

	/**
	 * Saves the content of the model to the file.
	 * @param path
	 */
	public void save(URI path) {
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
        options.put(org.eclipse.emf.ecore.xmi.XMLResource.OPTION_ENCODING, "UTF-8");

		try {
            resource.save(options);
        }
        catch (IOException e) {
            // TODO Auto-generated catch block
            //e.printStackTrace();
        }
	}

	/**
	 * Gets the ModuleEjbDefinition.
	 * @return
	 */
	public ModuleEjbDefinition getModel() {
		if (null == module) {
			EList l = resource.getContents();
			Iterator i = l.iterator();
			while (i.hasNext()) {
				Object o = i.next();
				if (o instanceof ModuleEjbDefinition)
					module = (ModuleEjbDefinition) o;
			}
		}
		return module;
	}
    /**
     * @return Returns the resource.
     */
    public   Resource getResource() {
        return resource;
    }
    /**
     * @param resource The resource to set.
     */
    public   void setResource(Resource resource) {
        this.resource = resource;
    }
	/**
	 * @return Returns the page.
	 */
	public static IWorkbenchPage getPage() {
		return page;
	}
	/**
	 * @param page The page to set.
	 */
	public void setPage(IWorkbenchPage page) {
		this.page = page;
		
	}
}
