package org.objectweb.jinha.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.objectweb.jinha.model.emf.jinha.GraphicalEJB;
import org.objectweb.jinha.model.emf.jinha.GraphicalEntity;
import org.objectweb.jinha.model.properties.PropertyEjbBeanEntity;

import com.objectlearn.lomboz.xml.lomboz.EntityBean;


/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class EntityModel extends DiagramElementModel
{
	private BusinessMethodListModel businessmethods;
	private AttributeListModel attributes;
    private HomeMethodListModel homeMethods;
    private CMPListModel CMPs;
    private CMRListModel CMRs;
    private FinderListModel finders;
	
    private String tableName;
    private String dataSourceName;
    // Reference vers model EMF SessionBean
    
    
	private List sourceConnections;
	private List targetConnections;

	private static IPropertyDescriptor[] descriptors;

	
	public static final String PROPERTY_SOURCE_CONNECTIONS_CHANGED = "source connections changed";
	public static final String PROPERTY_TARGET_CONNECTIONS_CHANGED = "target connections changed";

	public static final String ID_PROPERTY_TABLE_NAME = "table Name";
    private static final String ID_PROPERTY_DATASOURCE_NAME = "data source name";

    private PropertyEjbBeanEntity propertyEjbBeanEntity;
	/**
	 * 
	 */
	
    public EntityModel(String name,Resource res)
    {
        
        sourceConnections = new ArrayList();
        targetConnections = new ArrayList();
       
        factory.setTargetClass(GraphicalEntity.class);
        factory.setRes(res);
        graphicalEjb=(GraphicalEJB)factory.getNewObject();
        businessmethods = new BusinessMethodListModel();
        businessmethods.setEjb(graphicalEjb);
        attributes = new AttributeListModel();
        attributes.setEjb(graphicalEjb);
        homeMethods=new HomeMethodListModel();
        homeMethods.setEjb(graphicalEjb);
        CMPs=new CMPListModel();
        CMPs.setEjb(graphicalEjb);
        CMRs=new CMRListModel();
        CMRs.setEjb(graphicalEjb);
        finders=new FinderListModel();
        finders.setEjb(graphicalEjb);
        
        propertyEjbBeanEntity=new PropertyEjbBeanEntity(this,name);
       // propertyEjbRefSession=new PropertyEjbRefSession(graphicalEjb);
    
        setPackageName("defaultPackage");
        setTableName("");
        setDataSourceName("");
    }
	
	/**
	 * @return
	 */
	public BusinessMethodListModel getBusinessMethodsList()
	{
		return businessmethods;
	}
	/**
	 * @return
	 */
	public AttributeListModel getAttributeList()
	{
		return attributes;
	}
	/**
	 * @param ml
	 */
	public void setMethodList(BusinessMethodListModel ml)
	{
		this.businessmethods = ml;
	}
	/**
	 * @param al
	 */
	public void setAttributeList(AttributeListModel al)
	{
		this.attributes = al;
	}

	/**
	 * @param connection
	 */
	protected void addSourceConnection(RelationModel connection)
	{
		sourceConnections.add(connection);
		firePropertyChange(PROPERTY_SOURCE_CONNECTIONS_CHANGED, null, connection);
	}
	/**
	 * @param connection
	 */
	protected void addTargetConnection(RelationModel connection)
	{
		targetConnections.add(connection);
		firePropertyChange(PROPERTY_TARGET_CONNECTIONS_CHANGED, null, connection);
	}
	/**
	 * @param connection
	 */
	protected void removeSourceConnection(RelationModel connection)
	{
		sourceConnections.remove(connection);
		firePropertyChange(PROPERTY_SOURCE_CONNECTIONS_CHANGED, null, connection);
	}
	/**
	 * @param connection
	 */
	protected void removeTargetConnection(RelationModel connection)
	{
		targetConnections.remove(connection);
		firePropertyChange(PROPERTY_TARGET_CONNECTIONS_CHANGED, null, connection);
	}
	/**
	 * @return
	 */
	public List getSourceConnections()
	{
		return sourceConnections;
	}
	/**
	 * @return
	 */
	public List getTargetConnections()
	{
		return targetConnections;
	}
	/**
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null)
		{
            descriptors = (new IPropertyDescriptor[] {
                    new PropertyDescriptor("ejb.bean","ejb.bean"),
                    new TextPropertyDescriptor(ID_PROPERTY_PACKAGE, ID_PROPERTY_PACKAGE),
                    new TextPropertyDescriptor(ID_PROPERTY_TABLE_NAME,ID_PROPERTY_TABLE_NAME),
                    new TextPropertyDescriptor(ID_PROPERTY_DATASOURCE_NAME,ID_PROPERTY_DATASOURCE_NAME)
            });
            
             
           
		}
		return descriptors;
	}
	/**
	 * @param propName
	 * @param val
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object propName, Object val)
	{
        if (((String) propName).equals(ID_PROPERTY_PACKAGE))
            setPackageName((String) val);
        else if (((String) propName).equals(ID_PROPERTY_TABLE_NAME))
            setTableName((String) val);
        else if (((String) propName).equals(ID_PROPERTY_DATASOURCE_NAME))
            setDataSourceName((String) val);
        else
			super.setPropertyValue(propName, val);
	}
	/**
	 * @param propName
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object propName)
	{
		if(((String) propName).equals("ejb.bean"))
		    return  getPropertyEjbBeanEntity();
        else if (((String) propName).equals(ID_PROPERTY_PACKAGE))
            return getPackageName();
        else if (((String) propName).equals(ID_PROPERTY_TABLE_NAME))
            return getTableName();
        else if (((String) propName).equals(ID_PROPERTY_DATASOURCE_NAME))
            return getDataSourceName();
		else
			return super.getPropertyValue(propName);
	}
    /**
     * @return Returns the cMPs.
     */
    public CMPListModel getCMPs() {
        return CMPs;
    }
    /**
     * @param ps The cMPs to set.
     */
    public void setCMPs(CMPListModel ps) {
        CMPs = ps;
    }
    /**
     * @return Returns the cMRs.
     */
    public CMRListModel getCMRs() {
        return CMRs;
    }
    /**
     * @param rs The cMRs to set.
     */
    public void setCMRs(CMRListModel rs) {
        CMRs = rs;
    }
    /**
     * @return Returns the finders.
     */
    public FinderListModel getFinders() {
        return finders;
    }
    /**
     * @param finders The finders to set.
     */
    public void setFinders(FinderListModel finders) {
        this.finders = finders;
    }
    /**
     * @return Returns the homeMethods.
     */
    public HomeMethodListModel getHomeMethods() {
        return homeMethods;
    }
    /**
     * @param homeMethods The homeMethods to set.
     */
    public void setHomeMethods(HomeMethodListModel homeMethods) {
        this.homeMethods = homeMethods;
    }
    
   
    
    /**
     * @return
     */
    public PropertyEjbBeanEntity getPropertyEjbBeanEntity() {
        return propertyEjbBeanEntity;
    }
    /**
     * @param propertyEjbBeanEntity
     */
    public void setPropertyEjbBeanEntity(
            PropertyEjbBeanEntity propertyEjbBeanEntity) {
        this.propertyEjbBeanEntity = propertyEjbBeanEntity;
    }
    
    public String getName() {
        return getPropertyEjbBeanEntity().getEjbBeanName();
    }
    
    /**
     * @return Returns the sessionBean contained in the graphicalSession
     *         instance.
     */
    public EntityBean getEntityBean() {
        return ((GraphicalEntity) getGraphicalEjb()).getEntityBean();
    }
    
    /**
     * @return
     * @see org.objectweb.jinha.model.DiagramElementModel#getPackageName()
     */
    public String getPackageName() {
        // TODO Auto-generated method stub
        return super.getPackageName();
    }
    /**
     * @param packageName
     * @see org.objectweb.jinha.model.DiagramElementModel#setPackageName(java.lang.String)
     */
    public void setPackageName(String packageName) {
        // TODO Auto-generated method stub
        super.setPackageName(packageName);
        getEntityBean().getEntityEjb().getEjbClass().setValue(packageName + "."+getEntityBean().getEntityEjb().getEjbClass().getUnqualifiedClassName());
        
    }
    
    /**
     * @return Returns the tableName.
     */
    public String getTableName() {
        return tableName;
    }
    /**
     * @param tableName The tableName to set.
     */
    public void setTableName(String tableName) {
        this.tableName = tableName;
        
        // EMF link
        getEntityBean().setTableName(getTableName());
        
    }
    /**
     * @return Returns the dataSourceName.
     */
    public String getDataSourceName() {
        return dataSourceName;
    }
    /**
     * @param dataSourceName The dataSourceName to set.
     */
    public void setDataSourceName(String dataSourceName) {
        this.dataSourceName = dataSourceName;
        
        // EMF link
        getEntityBean().setDataSourceName(getDataSourceName());
    }
    
}
