package org.objectweb.jinha.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import com.sun.java.xml.ns.j2ee.MethodType;

/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class FinderModel extends ObjectModel
{
	private String identifier;
	private String returnType;
	private String visibility;
	private List parametersTypes;
	private IPropertyDescriptor[] descriptors;
    private MethodType method=null;
    private String query=null;

	public static final String PROPERTY_SIGNATURE_CHANGED = "signature";
    public static final String PUBLIC = "public";
    public static final String PRIVATE = "private";
    public static final String PROTECTED = "protected";

	public static final String ID_PROPERTY_IDENTIFIER = "identifier";
	public static final String ID_PROPERTY_RETURN_TYPE = "return type";
	public static final String ID_PROPERTY_VISIBILITY = "visibility";
	public static final String ID_PROPERTY_QUERY = "query";

	/**
	 * 
	 */
	public FinderModel(Resource res)
	{
        factory.setTargetClass(MethodType.class);
        factory.setRes(res);
        method=(MethodType)factory.getNewObject();
		setSignature(PUBLIC, "Collection", "finder", new ArrayList());
		
		setQuery("");
	}
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parametersTypes
	 */
	public FinderModel(String visibility, String returnType, String identifier, List parametersTypes)
	{
		setSignature(visibility, returnType, identifier, parametersTypes);
		setQuery("");
	}
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parametersTypes
	 */
	public void setSignature(String visibility, String returnType, String identifier, List parametersTypes)
	{
		this.visibility = visibility;
		this.returnType = returnType;
		this.identifier = identifier;
		this.parametersTypes = parametersTypes;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @return
	 */
	public String getVisibility()
	{
		return visibility;
	}
	/**
	 * @return
	 */
	public String getReturnType()
	{
		return returnType;
	}
	/**
	 * @return
	 */
	public String getIdentifier()
	{
		return identifier;
	}
	/**
	 * @return
	 */
	public List getParametersTypes()
	{
		return parametersTypes;
	}
	/**
	 * @return
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		String txt = returnType + " " + identifier + "(";
		Iterator it = parametersTypes.iterator();
		if (it.hasNext())
			txt += it.next().toString();
		while (it.hasNext())
		{
			txt += " ," + it.next().toString();
		}
		txt += ")";
		return txt;
	}
	/**
	 * @param returnType
	 */
	public void setReturnType(String returnType)
	{
		this.returnType = returnType;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @param identifier
	 */
	public void setIdentifier(String identifier)
	{
		this.identifier = identifier;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @param visibility
	 */
	public void setVisibility(String visibility)
	{
        if (visibility.equals("public") || visibility.equals("protected") || visibility.equals("private"))
		{
			this.visibility = visibility;
			firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
		}
	}
	/**
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null)
		{
            descriptors = (new IPropertyDescriptor[] {
				new TextPropertyDescriptor(ID_PROPERTY_RETURN_TYPE, ID_PROPERTY_RETURN_TYPE),
				new TextPropertyDescriptor(ID_PROPERTY_IDENTIFIER, ID_PROPERTY_IDENTIFIER),
				new ComboBoxPropertyDescriptor(
					ID_PROPERTY_VISIBILITY,
					ID_PROPERTY_VISIBILITY,
					new String[] { "public", "protected", "private" }),
				new TextPropertyDescriptor(ID_PROPERTY_QUERY, ID_PROPERTY_QUERY)
					
            });
		}
		return descriptors;
	}
	/**
	 * @param propName
	 * @param val
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object propName, Object val)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
				setReturnType(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				setIdentifier(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_QUERY))
				setQuery(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (((Integer) val).intValue() == 0)
					setVisibility(PUBLIC);
				else if (((Integer) val).intValue() == 2)
					setVisibility(PRIVATE);
				else if (((Integer) val).intValue() == 1)
					setVisibility(PROTECTED);
			}
		}
		else
			super.setPropertyValue(propName, val);
	}
	/**
	 * @param propName
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object propName)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
				return returnType;
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				return identifier;
			else if (((String) propName).equals(ID_PROPERTY_QUERY))
				return getQuery();
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (visibility == PUBLIC)
					return new Integer(0);
				else if (visibility == PRIVATE)
					return new Integer(2);
				else if (visibility == PROTECTED)
					return new Integer(1);
				else
					return super.getPropertyValue(propName);
			}
			else
				return super.getPropertyValue(propName);
		}
		else
			return super.getPropertyValue(propName);
	}
    /**
     * @return Returns the query.
     */
    public String getQuery() {
        return query;
    }
    /**
     * @param query The query to set.
     */
    public void setQuery(String query) {
        this.query = query;
    }
}