package org.objectweb.jinha.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.Bendpoint;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class RelationModel extends ObjectModel
{
	private EntityModel source;
	private EntityModel target;
    private CMRModel CMRsrc;
    private CMRModel CMRtarget;
    private String relationshipName;

    
    private static IPropertyDescriptor[] descriptors;

    public static final String           PROPERTY_SOURCE_DIRECTION_CHANGED = "Relation direction changed";
    public static final String           PROPERTY_SOURCE_RELATIONSHIP_NAME = "name";
    
    private int                          direction;
    /**
     * @return Returns the cMRsrc.
     */
    public CMRModel getCMRsrc() {
        return CMRsrc;
    }
    /**
     * @param rsrc The cMRsrc to set.
     */
    public void setCMRsrc(CMRModel rsrc) {
        CMRsrc = rsrc;
    }
    /**
     * @return Returns the cMRtarget.
     */
    public CMRModel getCMRtarget() {
        return CMRtarget;
    }
    /**
     * @param rtarget The cMRtarget to set.
     */
    public void setCMRtarget(CMRModel rtarget) {
        CMRtarget = rtarget;
    }
	private List bendpoints;

	/**
	 * Comment for <code>PROPERTY_CONSTRAINT_CHANGED</code>
	 */
	public static final String PROPERTY_CONSTRAINT_CHANGED = "constraint changed";

    private static final String PROPERTY_SOURCE_DIRECTION   = "direction";
    private Resource res=null;
	/**
	 * 
	 */
	public RelationModel(Resource res)
	{
		factory.setRes(res);
		//factory.setTargetClass(RelationshipsType);
		bendpoints = new ArrayList();
        this.res=res;
        
        setRelationshipName("");
	}
	public void setSource(EntityModel newSource) {
        if (source == newSource)
            return;

        EntityModel oldSource = source;
        source = newSource;
        if (oldSource != null)
            oldSource.removeSourceConnection(this);
        if (newSource != null)
            source.addSourceConnection(this);
    }

    public void setTarget(EntityModel newTarget) {
        if (target == newTarget)
            return;

        EntityModel oldTarget = target;
        target = newTarget;
        if (oldTarget != null)
            oldTarget.removeTargetConnection(this);
        if (newTarget != null)
            target.addTargetConnection(this);
    }

    public EntityModel getSource() {
        return source;
    }

    public EntityModel getTarget() {
        return target;
    }

    public List getConstraint() {
        return bendpoints;
    }

    public void setConstraint(List bendpoints) {
        this.bendpoints = bendpoints;
        firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, null, null);
    }

    public void addBendpoint(Bendpoint bendpoint, int index) {
        bendpoints.add(index, bendpoint);
        firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, null, null);
    }

    public void removeBendpoint(Bendpoint bendpoint) {
        bendpoints.remove(bendpoint);
        firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, null, null);
    }

    public Bendpoint removeBendpoint(int index) {
        Bendpoint save = (Bendpoint) bendpoints.get(index);
        bendpoints.remove(index);
        firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, null, null);
        return save;
    }

    public Bendpoint getBendpoint(int index) {
        return (Bendpoint) bendpoints.get(index);
    }

    public void replaceBendpoint(int index, Bendpoint newBendpoint) {
        bendpoints.remove(index);
        bendpoints.add(index, newBendpoint);
        firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, null, null);
    }

    public String getSourceFullName() {
        return getSource().getName();
    }

    public String getTargetFullName() {
        return getTarget().getName();

    }

    public String targetToSource() {
        return getTargetFullName() + " to " + getSourceFullName();
    }

    public String sourceToTarget() {
        return getSourceFullName() + " to " + getTargetFullName();
    }

    public String[] getListStringChoice() {

        return new String[] { sourceToTarget(), targetToSource(),
                "Bidirectionnal" };

    }

    /**
     * @return Returns the direction.
     */
    public int getDirection() {
        return direction;
    }

    /**
     * @param direction
     *            The direction to set.
     */
    public void setDirection(int direction) {

        int save = this.direction;
        this.direction = direction;

        if (save != direction) {
            applyDirectionChanged();
            firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, null, null);
        }

    }

    public IPropertyDescriptor[] getPropertyDescriptors() {

        descriptors = (new IPropertyDescriptor[] {

        new ComboBoxPropertyDescriptor(PROPERTY_SOURCE_DIRECTION, PROPERTY_SOURCE_DIRECTION,
                getListStringChoice()),

           new TextPropertyDescriptor(PROPERTY_SOURCE_RELATIONSHIP_NAME,PROPERTY_SOURCE_RELATIONSHIP_NAME)     
        });
        return descriptors;

    }

    public void setPropertyValue(Object propName, Object val) {
        if (((String) propName).equals(PROPERTY_SOURCE_DIRECTION)) {
            setDirection(((Integer) val).intValue());

        }
        else if (((String) propName).equals(PROPERTY_SOURCE_RELATIONSHIP_NAME)) {
            setRelationshipName(((String) val));

        }
        super.setPropertyValue(propName, val);
    }

    public Object getPropertyValue(Object propName) {

        if (((String) propName).equals(PROPERTY_SOURCE_DIRECTION))
            return new Integer(getDirection());
        else if (((String) propName).equals(PROPERTY_SOURCE_RELATIONSHIP_NAME))
            return getRelationshipName();
        else
            return super.getPropertyValue(propName);
    }


    public void applyDirectionChanged() {

        switch (direction) {

            case 0: // Case source to target

                // Need to remove a possible existing CMR from target to source
                if (CMRtarget != null) {
                    target.getCMRs().removeElement(CMRtarget);
                   
                    CMRtarget = null;
                }

                // create the CMR for this direction
                if (CMRsrc == null) {

                    CMRsrc = new CMRModel(target.getName(),res);
                    source.getCMRs().addElement(CMRsrc);
                }
                break;
            case 1:
                // Case target to source
                //  Need to remove a possible existing CMR from target to source
                if (CMRsrc != null) {
                    source.getCMRs().removeElement(CMRsrc);
                    CMRsrc = null;

                }

                if (CMRtarget == null) {
                    CMRtarget = new CMRModel(source.getName(),res);
                    target.getCMRs().addElement(CMRtarget);

                }
                break;
            case 2:
                // Case bidirectionnal
                if (CMRsrc == null) {
                    CMRsrc = new CMRModel(target.getName(),res);
                    source.getCMRs().addElement(CMRsrc);
                }

                if (CMRtarget == null) {
                    CMRtarget = new CMRModel(source.getName(),res);
                    target.getCMRs().addElement(CMRtarget);
                }

                break;
        }

    }
    /**
     * @return Returns the relationshipName.
     */
    public String getRelationshipName() {
        return relationshipName;
    }
    /**
     * @param relationshipName The relationshipName to set.
     */
    public void setRelationshipName(String relationshipName) {
        this.relationshipName = relationshipName;
    }
}
